package com.persagy.meiku.services

import com.persagy.database.SFilter
import com.persagy.meiku.models.entities.MeiKuAnchor
import com.persagy.meiku.models.entities.PipelineCategory
import com.persagy.mybatis.SMybatisDao
import com.persagy.service.SObjectService
import com.persagy.service.models.enums.SResponseType
import com.persagy.service.models.responses.SQueryResponse
import java.util.*

/**
 * 管线服务
 *
 * @author wx  <zhangweixin@sagacloud.com>
 * @date  2020/11/5 10:17
 */
object PipelineCategoryService: SObjectService<PipelineCategory>(SMybatisDao(PipelineCategory::class.java)) {

    /**
     * 查询管线类型
     */
    fun categoryQuery(): SQueryResponse<PipelineCategory>  {
        return try {
            val sQueryResponse = SQueryResponse<PipelineCategory>()
            val pipelineCategoryList = select(SFilter.isNull("parentId")).exec()
            for (pipelineCategory in pipelineCategoryList){
                pipelineCategory.categoryList = select(SFilter.eq("parentId",pipelineCategory.id!!)).exec()
            }
            sQueryResponse.content = pipelineCategoryList
            sQueryResponse.total = pipelineCategoryList.size.toLong()
            sQueryResponse.result = SResponseType.success
            sQueryResponse
        } catch (e: Exception) {
            e.printStackTrace()
            SQueryResponse(SResponseType.failure,e.message!!)
        }
    }

    override fun onCreateBefore(entity: PipelineCategory): Boolean {
        if (entity.id.isNullOrEmpty()){
            entity.id = UUID.randomUUID().toString().replace("-", "")
        }
        return super.onCreateBefore(entity)
    }

}